package at.jku.ssw.graph;

import at.jku.ssw.Iterator;
import at.jku.ssw.list.List;
import at.jku.ssw.list.impl.LinkedList;

/** A graph of character vertices. */
public class Graph implements Cloneable {

	public final List vertices;
	public List edges;

	/** Creates an empty graph. */
	public Graph() {
		vertices = new LinkedList();
		edges = new LinkedList();
	}

	public void reset() {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			v.reset();
		}
	}

	/**
	 * Creates a {@link #reset() reseted} copy of this class.
	 */
	@Override
	public Graph clone() {
		Graph cloned = new Graph();
		Iterator vertexIt = vertices.iterator();
		while (vertexIt.hasNext()) {
			Vertex vertex = (Vertex) vertexIt.next();
			cloned.vertices.insert(cloned.vertices.size(), vertex.clone());
		}
		Iterator edgeIt = edges.iterator();
		while (edgeIt.hasNext()) {
			Edge edge = (Edge) edgeIt.next();
			cloned.edges.insert(cloned.edges.size(), edge.clone());
		}
		return cloned;
	}

	/** Adds a vertex to the graph. */
	public void addVertex(Vertex vertex) {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			if (vertex.value == v.value) {
				throw new IllegalArgumentException("Vertex already exists");
			}
		}
		if (vertex.isRoot && getRoot() != null) {
			throw new IllegalArgumentException("Root already exists");
		}
		vertices.insert(vertices.size(), vertex);
	}

	/** Returns the root vertex of the graph. */
	public Vertex getRoot() {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			if (v.isRoot) {
				return v;
			}
		}
		return null;
	}

	/** Adds an edge to the graph. */
	public void addEdge(Edge edge) {
		checkVertex(edge.start);
		checkVertex(edge.end);
		for (int i = 0; i < edges.size(); i++) {
			Edge e = (Edge) edges.get(i);
			if (isEqual(e, edge)) {
				throw new IllegalArgumentException("Edge already exists");
			}
		}
		edges.insert(edges.size(), edge);
	}

	private boolean isEqual(Edge e1, Edge e2) {
		return (e1.start == e2.start && e1.end == e2.end) || ((e1.start == e2.end && e1.end == e2.start
				&& (e1.kind == Edge.Kind.Undirected || e2.kind == Edge.Kind.Undirected)));
	}

	private void checkVertex(Vertex vertex) {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			if (v == vertex) {
				return;
			}
		}
		throw new IllegalArgumentException("Vertex does not exist");
	}

	/**
	 * Returns an iterator containing the outgoing edges for a vertex.
	 * 
	 * All elements {@link Iterator#next() returned} by this iterator are of
	 * type {@link Edge}.
	 */
	public Iterator getEdgeIterator(Vertex vertex) {
		LinkedList result = new LinkedList();
		for (int i = 0; i < edges.size(); i++) {
			Edge e = (Edge) edges.get(i);
			if (e.start == vertex) {
				result.insert(result.size(), e);
			} else if (e.kind == Edge.Kind.Undirected && e.end == vertex) {
				result.insert(result.size(), new Edge(e.end, e.start, e.weight, e.kind));
			}
		}
		return result.iterator();
	}

	/** Returns the edge between the start and the end vertex. */
	public Edge getEdge(Vertex start, Vertex end) {
		for (int i = 0; i < edges.size(); i++) {
			Edge e = (Edge) edges.get(i);
			if (isEqual(e, new Edge(start, end, Edge.Kind.Directed))) {
				return e;
			}
		}
		return null;
	}

}
